@@ -240,7 +240,38 @@ proc CmdCopyAs_ { listb_name filelist_var frompwd topwd } {
   }
 }
 
+proc SoftLink { src dst } {
+  global config
+  if {$config(create_relative_links)} {
+    set srcdir [file dirname $src]
+    set dstdir [file dirname $dst]
+    set srcfile [file tail $src]
+    set dstfile [file tail $dst]
+    set dstlist [split $dstdir /]
+    set srclist [split $srcdir /]
+    set dstlen [llength $dstlist]
+    set srclen [llength $srclist]
+    # Count how many directories are the same in the source and destination paths
+    set index 0
+    while {([lindex $srclist $index] == [lindex $dstlist $index]) && ($index < $srclen) && ($index < $dstlen)} {
+      incr index
+    }
+    # Build relative link
+    set link {}
+    for {set dstindex $index} {$dstindex < $dstlen} {incr dstindex} {
+      append link ../
+    }
+    for {set srcindex $index} {$srcindex < $srclen} {incr srcindex} {
+      append link [lindex $srclist $srcindex]/
+    }
+    set from $link$srcfile
 
+    #puts "$src $dst $srcdir $srcfile $dstdir $dstfile : ln -s $from $dst"
+    Try { exec $config(cmd,ln) -s "$from" "$dst" } "" 1 
+  } else {
+    Try { exec $config(cmd,ln) -s "$src" "$dst" } "" 1 
+  }
+}
 
 proc CmdSoftLink {} {
   global glob
@@ -272,7 +303,7 @@ proc CmdSoftLink_ { listb_name filelist_var frompwd topwd } {
       l   -
       ld  { 
         Log "Softlinking $frompwd/[lindex $elem 1] to $topwd"
-        Try { exec $config(cmd,ln) -s "$frompwd/[lindex $elem 1]" "$topwd" } "" 1 
+        SoftLink $frompwd/[lindex $elem 1] $topwd/[lindex $elem 1]
       }
       default CantDoThat
     }
@@ -310,7 +341,8 @@ proc CmdSoftLinkAs_ { listb_name filelist_var frompwd topwd } {
         }
         if {$destfile != "" } {
           Log "Softlinking $frompwd/[lindex $elem 1] to $destfile"
-          Try { exec $config(cmd,ln) -s "$frompwd/[lindex $elem 1]" "$destfile" } "" 1 }
+          SoftLink $frompwd/[lindex $elem 1] $destfile
+        }
       }
       default CantDoThat
     }
@@ -907,6 +939,94 @@ proc CmdRunCmd_ { listb_name filelist_var frompwd topwd } {
   UpdateWindow both
 }
 
+proc CmdForEach {} {
+  global glob
+  CmdForEach_ $glob(win,left).frame_listb.listbox1 glob(left,filelist) $glob(left,pwd) $glob(right,pwd) 
+  CmdForEach_ $glob(win,right).frame_listb.listbox1 glob(right,filelist) $glob(right,pwd) $glob(left,pwd) 
+}
+
+proc CmdForEach_ { listb_name filelist_var frompwd topwd } {
+  global glob config
+  upvar $filelist_var filelist
+
+  set fl {}
+  foreach sel [$listb_name curselection] {
+    set elem [lindex $filelist $sel]
+    switch [lindex $elem 2] {
+      d   -
+      ld  -
+      l   -
+      n   { lappend fl [lindex $elem 1] }
+      default CantDoThat
+    }
+  }
+  if { $fl == "" } return
+  if {$glob(async)} {
+    PopError "This command does not support asynchronous execution"
+    return
+  }
+  if {![info exists glob(foreach,cmd)] || $glob(foreach,cmd) == {}} {
+    set glob(foreach,cmd) {echo '%s'}
+  }
+  set glob(foreach,cmd) [EntryDialog "Foreach" "Enter command to run on each of the selected files. The file will show up in the '%s'. You can use pipes and other bourne-shell syntax elements since the commands will each run in a separate bourne shell." $glob(foreach,cmd)]
+  if { $glob(foreach,cmd) == "" } return
+
+  set output {}
+  foreach k $fl {
+    if {[CheckAbort "ForEach"]} return
+    set cmd [format "$glob(foreach,cmd)" $k]
+    append output "\$ $cmd\n"
+    LogStatusOnly "Running $cmd ..."
+    catch { cd $frompwd ; exec $config(cmd,sh) -c "$cmd" } out
+    append output "$out\n"
+  }
+  ViewString "Output from commands" output ""
+  UpdateWindow both
+}
+
+proc CmdRecurseCommand { inst } {
+  global glob config
+
+  set dir $glob($inst,pwd)
+
+  if { [IsFTP $dir] } {
+    CantDoThat
+    return
+  }
+
+  if {$glob(async)} {
+    PopError "This command does not support asynchronous execution"
+    return
+  }
+  if {![info exists glob(foreach,cmd)] || $glob(foreach,cmd) == {}} {
+    set glob(foreach,cmd) {echo '%s'}
+  }
+  set e [EntryDialogDouble "Run Command Recursively" "Enter command to run on each file. The file will show up in the '%s'. You can use pipes and other bourne-shell syntax elements since the commands will each run in a separate bourne shell." "Enter pattern to match for when recursing into directories." "This command will run a 'find' of the type\n\n    $ find <directory> -type f -name <pattern> -print\n\nto recurse into the current directory ($dir). The command will then be run on all files from this find.\n\nSee also: manpage for the find command.\n\nUse tab to go to next entry. Return or the OK button starts execution." $glob(foreach,cmd) {*}]
+  if { $e == "" } return
+
+  set glob(foreach,cmd) [lindex $e 0]
+  set pattern [lindex $e 1]
+  set output {}
+
+  # expand file list with find outputs
+  set r [catch {cd $dir ; exec $config(cmd,find) . -type f -name $pattern -print} out]
+  if {$r} {
+    PopError "$out"
+    return
+  }
+  set fl [split $out \n]
+  foreach k $fl {
+    if {[CheckAbort "Recurse Command"]} return
+    set cmd [format "$glob(foreach,cmd)" $k]
+    append output "\$ $cmd\n"
+    LogStatusOnly "Running $cmd ..."
+    catch { cd $dir ; exec $config(cmd,sh) -c "$cmd" } out
+    append output "$out\n"
+  }
+  ViewString "Output from commands" output ""
+  UpdateWindow both
+}
+
 
 proc CmdDiff {} {
   global glob
@@ -1154,7 +1274,10 @@ proc CmdGetHttp { inst } {
     return
   }
 
-  set URL {}
+  if {![info exists glob(http,lasturl)]} {
+    set glob(http,lasturl) {}
+  }
+  set URL $glob(http,lasturl)
 
   while { 1 } {
     set URL [EntryDialog "Get HTTP File" "Please enter HTTP URL to download" $URL]
@@ -1174,6 +1297,7 @@ proc CmdGetHttp { inst } {
     if {$location == ""} {
       append URL /
     }
+    set glob(http,lasturl) $URL
     break
   }
 
@@ -1189,3 +1313,4 @@ proc CmdGetHttp { inst } {
   HTTP_Get $URL $filename
   UpdateWindow both
 }
+
