@@ -125,9 +125,9 @@ GetTimeFromSecs(ClientData clientData, Tcl_Interp* interp,
   time_s = localtime(&t);
   if (dateformat)
     sprintf(interp->result, "%02d%02d%02d %02d:%02d:%02d", time_s->tm_mday, time_s->tm_mon+1,
-          time_s->tm_year, time_s->tm_hour, time_s->tm_min, time_s->tm_sec);
+          time_s->tm_year % 100, time_s->tm_hour, time_s->tm_min, time_s->tm_sec);
   else
-    sprintf(interp->result, "%02d%02d%02d %02d:%02d:%02d", time_s->tm_year, time_s->tm_mon+1,
+    sprintf(interp->result, "%02d%02d%02d %02d:%02d:%02d", time_s->tm_year % 100, time_s->tm_mon+1,
           time_s->tm_mday, time_s->tm_hour, time_s->tm_min, time_s->tm_sec);
   return TCL_OK;
 }
